// (default) options used in this infoprovider
// Note: call setOptions() before doing anything!
var options;

function setOptions(values) {
	options = values;
}

// initialize appcontroller for
// json or xml request, caching as well as message to pebble send operations
// Note: call setAppController() before doing anything!
var appcontroller;

function setAppController(controller) {
	appcontroller = controller;

	if (appcontroller.pebblehelpers.isEmery()) {
		MAX_CONTENT_LENGTH = 219;
	}
}

/*
 * generic module interace
 */

module.exports = {
		// set (default) options used in this module
		setOptions : setOptions,

		// set app controller used in this module
		setAppController : setAppController,
};



// module properties
var MAX_NEWS_PROVIDERS = 15;  // determines the max news providers configurable by settings
var MAX_FEED_ENTRIES   = 24;
var MAX_CONTENT_LENGTH = 159; // is changed for emery platform in setAppController()


var rssFeeds = new Array(
		"https://news.google.com/?output=rss", // Google News EN
		"https://news.google.de/?output=rss", // Google News DE
		"https://news.google.fr/?output=rss", // Google News FR
		"https://news.google.pt/?output=rss", // Google News PT
		"https://news.google.it/?output=rss", // Google News IT
		"https://news.google.cz/?output=rss", // Google News CZ
		"http://rss.nytimes.com/services/xml/rss/nyt/HomePage.xml", // NY Times homepage
		"http://rss.nytimes.com/services/xml/rss/nyt/MostViewed.xml", // NY Times most viewed
		"http://rss.nytimes.com/services/xml/rss/nyt/Economy.xml", // NY Times economy
		"http://dealbook.nytimes.com/feed/", // NY Times dealbook
		"http://rss.nytimes.com/services/xml/rss/nyt/YourMoney.xml", // NY Times your money
		"http://rss.nytimes.com/services/xml/rss/nyt/Business.xml", // NY Times business
		"http://rss.nytimes.com/services/xml/rss/nyt/Sports.xml", // NY Times sport
		"http://rss.nytimes.com/services/xml/rss/nyt/Tennis.xml", // NY Times tennis
		"http://rss.nytimes.com/services/xml/rss/nyt/World.xml", // NY Times world
		"http://www.stern.de/feed/standard/all/", // stern - der Tag
		"http://www.stern.de/feed/daily/", // stern - Bilder des Tages
		"http://www.stern.de/feed/standard/alle-nachrichten/", // stern - Nachrichten
		"http://www.stern.de/feed/standard/sport/", // stern - Sport
		"http://www.stern.de/feed/standard/wirtschaft/", // stern - Wirtschaft
		"http://www.heise.de/developer/rss/news-atom.xml", // heise developer news
		"http://www.heise.de/mac-and-i/news-atom.xml", // heise mac & i
		"http://www.npr.org/rss/rss.php?id=1001", // NPR News
		"http://www.npr.org/rss/rss.php?id=1006", // NPR Business
		"http://www.npr.org/rss/rss.php?id=1003", // NPR US News
		"http://www.npr.org/rss/rss.php?id=1004", // NPR World News
		"http://www.lefigaro.fr/rss/figaro_politique.xml", // le Figaro politique
		"http://www.lefigaro.fr/rss/figaro_hightech.xml" // le Figaro hightech
       );



// module functions

/*
 * transfer properties into a better structure; handle not set properties
 */

function getNewsConfigurations() {
    var newsConfigurations = new Array();
    var counter = 0;

    for (var i = 1; i < MAX_NEWS_PROVIDERS; i++) {
        var index = options["newsProvider" + i];
        if (typeof index === "undefined" || index == 0) {
            // nothing remains to do
            continue;
        }

        newsConfigurations[counter] = { "newsProvider": index };

        var myFeed = options["myFeed" + i];
        if (typeof myFeed !== "undefined") {
            newsConfigurations[counter]["myFeed"] = myFeed;
        }
        var googleSearch = options["googleSearch" + i];
        if (typeof googleSearch !== "undefined") {
            newsConfigurations[counter]["googleSearch"] = googleSearch;
        }
        counter++;
    }

    return newsConfigurations;
}

/*
 * central send news method with a lot of preprocessing done before
 */

function sendNewsMessage(providerID, index, title, content) {
	// a little bit preprocessing
	title   = appcontroller.pebblehelpers.getDecodedTrimedString(title);
	content = appcontroller.pebblehelpers.getDecodedTrimedString(content);

	// get news text
	var newsText = title;
	if (title.length>0 && content.length>0) {
		newsText += ": ";
	}
	newsText += content;

    // Debug content - begin
	//newsText = "EU-Kommissar Oettinger hat sich für seine Äußerungen während einer Rede in Hamburg entschuldigt - nach umfangreichen Protesten und Druck von Kommissionsöä";
    // Debug content - end

	// do a little postprocessing to ensure that no half letters are transfered to pebble
	newsText = newsText.substr(0,MAX_CONTENT_LENGTH);

	var byteLength = appcontroller.pebblehelpers.getByteLength(newsText);
	var charLength = newsText.length;

	var diff = byteLength - charLength;
	//console.log( "Text has " + byteLength + " bytes and " + charLength + " chars -> diff is " + diff );

	if(diff > 0) {
	    //console.log( "will transfer only " + (MAX_CONTENT_LENGTH - diff - 1) + " chars!" );
		newsText = newsText.substr(0,MAX_CONTENT_LENGTH - diff - 1);
		//console.log("resulting text >" + newsText + "<");
	}

    if(newsText.length > 0) {
    	if (isFeedTest) {
    		// for test purposes
    		console.log(providerID + ": " + index + ".) " + newsText);
    	}
    	else {
	        appcontroller.appMessageQueue.push({
	            "message" : {
	                "providerID" : providerID,
	                "index" : index,
	                "newsContent" : newsText
	            }
	        });
    	}
    }
}

/*
 * central generic parse feed method
 */

// helper method for parseFeed()
function getElementValue(element, key) {
    if (typeof element[key] != "undefined") {
        return (typeof element[key][0]["_"] != "undefined") ? element[key][0]["_"] : element[key][0];
    }

    return "";
}

function parseFeed(providerID, tmpResult) {
    var result = JSON.parse(JSON.stringify(tmpResult));
    var index = 0;

    if (typeof result["feed"] != "undefined") {
        // it's an atom feed
        result["feed"]["entry"].forEach(function(element1, index1, array1) {
            if ( index < MAX_FEED_ENTRIES / options.providerCount ) {
                if (typeof element1["title"] != "undefined" ||
                    typeof element1["summary"] != "undefined" ||
                    typeof element1["content"] != "undefined") {

                    var title  = getElementValue(element1, "title");

                    var summary1 = getElementValue(element1, "summary");
                    var summary2 = getElementValue(element1, "content");

                    var content = "";
                    if (summary1.length > 0) {
                        content += summary1;
                    }
                    else if (summary2.length > 0) {
                        content += summary2;
                    }

                    sendNewsMessage(providerID, index++, title, content);
                }
            }
        });
    }
    else {
        // it's a (special) rss feed
        appcontroller.pebblehelpers.getValuesByKeyName(result, "item").forEach(function(element1, index1, array1) {
            element1.forEach(function(element2, index2, array2) {
                if ( index < MAX_FEED_ENTRIES / options.providerCount ) {
                    if (typeof element2["title"] != "undefined") {
                        var title   = appcontroller.pebblehelpers.isArray(element2["title"]) ? element2["title"][0] : element2["title"];

                        var content = "";
                        if (typeof element2["description"] != "undefined") {
                            content += element2["description"];
                        }

                        sendNewsMessage(providerID, index++, title, content);
                    }
                }
            });
        });
    }
}

function fetchRSS(providerID, url) {
	appcontroller.startXMLRequest(url,
        function(err, result) {
            //console.log("*** feed: " + url + ":\n" + JSON.stringify(result));
            try {
                parseFeed(providerID, result);
            }
            catch (error) {
                console.log("Error during parsing RSS/ ATOM feed! [feed: " + url + "]\n" + error.stack);
                appcontroller.appMessageQueue.push({
                    "message" : {
                        "error" : "feed error"
                    }
                });
            }
        }
    );
}

/* disabled */
function fetchGoogleSearch(providerID, query) {
    var url = "https://ajax.googleapis.com/ajax/services/search/news?v=1.0&q=" + encodeURI(query);

    appcontroller.startJSONRequest(url,
        function(result) {
            var index = 0;
            if (result != null && result.responseData != null && result.responseStatus != 404 && result.responseData.results.length > 0) {
                result.responseData.results.forEach(function(element, index1, array) {
                    if ( index < MAX_FEED_ENTRIES / options.providerCount ) {
                        if (typeof element.title != "undefined") {
                            var title   = element.title;

                            var content = "";
                            if (typeof element.content != "undefined") {
                                content += element.content;
                            }

                            sendNewsMessage(providerID, index++, title, content);
                        }
                    }
                });
            }
            else {
                appcontroller.appMessageQueue.push({
                    "message" : {
                        "providerID" : providerID,
                        "index" : index++,
                        "newsContent" : "Got no search result for token \"" + query + "\" - sorry!"
                    }
                });
            }
        }
    );
}

function fetchNTVHome(providerID) {
    var url = "http://apps.n-tv.de/apps/common/?service=mobile&type=frontpage&ntv13=true";

    appcontroller.startXMLRequest(url,
        function(err, tmpResult) {
            var result = JSON.parse(JSON.stringify(tmpResult));
            var index = 0;
            result["ntv:feed"]["ntv13:group"].forEach(function(element1, index1, array1) {
                if (typeof element1["ntv:article"] != "undefined") {
                    element1["ntv:article"].forEach(function(element2, index2, array2) {
                        if ( index < MAX_FEED_ENTRIES / options.providerCount ) {
                            if (element2["$"]["ntv:contenttype"] === "meldung" && typeof element2["ntv:field"] != "undefined") {
                                var title   = element2["ntv:field"][0]._;

                                var content = "";
                                if (typeof element2["ntv:field"][1]._ != "undefined") {
                                    content += element2["ntv:field"][1]._;
                                }
                                else if (typeof element2["ntv:field"][2]._ != "undefined") {
                                    content += element2["ntv:field"][2]._;
                                }

                                sendNewsMessage(providerID, index++, title, content);
                            }
                        }
                    });
                }
            });
        }
    );
}

/* disabled */
function fetchNTV(providerID) {
    var url = "http://apps-cloud.n-tv.de/?service=samsung_rss";

    fetchRSS(providerID, url);
}

/* disabled */
function fetchBBC(providerID) {
    var url = "http://api.breakingnews.com/api/v1/item/";

    appcontroller.startJSONRequest(url,
        function(result) {
            var index = 0;
            result.objects.forEach(function(element, index1, array) {
                if ( index < MAX_FEED_ENTRIES / options.providerCount ) {
                    var title   = element.content;
                    var content = "";

                    sendNewsMessage(providerID, index++, title, content);
                }
            });
        }
    );
}

function fetchFaroo(providerID, query) {
    var url = "http://www.faroo.com/api?q=" + encodeURIComponent(query) + "&start=1&length=10&src=news&f=rss";

    fetchRSS(providerID, url);
}

function fetchNews(providerID) {
    var newsConfigurations = getNewsConfigurations();

    // check configuration
    var newsConfiguration = newsConfigurations[providerID];
    if (typeof newsConfiguration === "undefined") {
        appcontroller.appMessageQueue.push({
            "message" : {
                "nodata" : "no news#" + (providerID + 1)
            }
        });
        appcontroller.sendAppMessage(true);
        return;
    }

    var index  = newsConfiguration[ "newsProvider" ];

    // process request
    switch (index) {
        case 1:
            if (newsConfiguration[ "myFeed" ].replace(/(^\s+|\s+$)/g, '') == ""){
                appcontroller.appMessageQueue.push({
                    "message" : {
                        "nodata" : "empty feed#" + (providerID + 1)
                    }
                });
                appcontroller.sendAppMessage(true);
                return;
            }
            fetchRSS(providerID, newsConfiguration[ "myFeed" ]);
            break;
        case 2:
            if (newsConfiguration[ "googleSearch" ].replace(/(^\s+|\s+$)/g, '') == ""){
                appcontroller.appMessageQueue.push({
                    "message" : {
                        "nodata" : "empty search#" + (providerID + 1)
                    }
                });
                appcontroller.sendAppMessage(true);
                return;
            }
            // Note: google search API is down :(
            //fetchGoogleSearch(providerID, newsConfiguration[ "googleSearch" ]);
            fetchFaroo(providerID, newsConfiguration[ "googleSearch" ]);
            break;
        case 3:
            fetchNTVHome(providerID);
            break;
        default:
            var feedID = index - 3;

            if(feedID > rssFeeds.length) {
                // nothing remains to do
                console.log("Error - feed not defined [ feedID: " + feedID + "; length: " + rssFeeds.length + " ]");
            }
            else {
                fetchRSS(providerID, rssFeeds[feedID - 1]);
            }
            break;
    }
}

// only for test purposes
var isFeedTest = false;

function testAllFeeds() {
	if (isFeedTest) {
		// nothing remains to do
		return
	}

	isFeedTest = true;

	console.log("TEST: start");
	for (var index = 0; index < 4 + rssFeeds.length; index++) {
		// set dummy providerID
		var providerID = index;

		try {
		    switch (index) {
		    	case 0:
			    	console.log("none");
			    	break;
		    	case 1:
			    	console.log("my feed");
			    	break;
		    	case 2:
			    	console.log("google search");
			    	break;
			    case 3:
			    	console.log("n-tv home");
			        fetchNTVHome(providerID);
			        break;
			    default:
			        var feedID = index - 3;

			        if(feedID > rssFeeds.length) {
			            // nothing remains to do
			            console.log("Error - feed not defined [ feedID: " + feedID + "; length: " + rssFeeds.length + " ]");
			        }
			        else {
				    	console.log("call feed ID: " + feedID);
			            fetchRSS(providerID, rssFeeds[feedID-1]);
			        }
			        break;
		    }
		}
		catch(error) {
		    console.log("Exception occurred in providerID " + providerID + ": " +  error.message);
		}
	}
	console.log("TEST: done");
}



/*
 * individual module interace
 */

module.exports.fetch        = fetchNews;
module.exports.testAllFeeds = testAllFeeds;
