// (default) options used in this infoprovider
// Note: call setOptions() before doing anything!
var options;

function setOptions(values) {
	options = values;
}

// initialize appcontroller for
// json or xml request, caching as well as message to pebble send operations
// Note: call setAppController() before doing anything!
var appcontroller;

function setAppController(controller) {
	appcontroller = controller;
}

var STOCKS_ID            = 'stocks';

/*
 * generic module interace
 */

module.exports = {
		// set (default) options used in this module
		setOptions : setOptions,

		// set app controller used in this module
		setAppController : setAppController,

		// ID used for caching module infos
		STOCKS_ID : STOCKS_ID
};



// module properties

// none



// module functions

/*
 * fetch stock operations
 */

// helper methods
function formatValues(values) {
    return values.replace(/\//g,"");
}

function getStockTokenString(element) {
    return element.symbol + "|" +
           element.price + "|" +
           (parseFloat(element.day_change)*100.0/parseFloat(element.price)).toFixed(1) + "|";
}

function fetchStockValues() {
	// Debug content - begin
	/*var urlArray   = ["https://www.worldtradingdata.com/api/v1/stock?symbol=AAPL,MSFT,HSBA.L&api_token=demo",
		              "https://www.worldtradingdata.com/api/v1/stock?symbol=AAPL,MSFT,HSBA.L&api_token=demo"];*/
	// Debug content - end
	var urlArray   = [];

	// free www.worldtradingdata.com version is limited to 5 stock values per request - due to this we need multi requests in case more than 5 stocks are requested
	// Note: max 25 request per day are supported
	var stockArray = options.stocks.split(",");
	var helpArray  = [];

	var URL = (options.apiToken == "demo" ? "https://api.worldtradingdata.com" : "https://www.worldtradingdata.com");

	stockArray.forEach(function(element, index1, array) {
		helpArray.push(element);
		index1++;

		if ( index1 % 5 == 0 || index1 == stockArray.length ) {
		    var query       = encodeURI(formatValues( helpArray.join(",") )).replace(/&/g,"%26");
		    urlArray.push( URL +
		    		       "/api/v1/stock?symbol=" + query +
		    		       "&sort_by=list_order&api_token=" + options.apiToken );
			helpArray       = [];
		}
	});
    //console.log(JSON.stringify(urlArray));

    // start request(s)
    var urlIndex     = 0;
    var index        = 0;
    var stockContent = [];

    urlArray.forEach(function(url, index1, array) {
	    appcontroller.startJSONRequest(url,
	        function(result) {
	    		urlIndex++; // got next result
	    		stockContent[index1] = "";

	    		//console.log(JSON.stringify(result));
	            if (result.symbols_returned > 0) {
	                result.data.forEach(function(element, index2, array) {
	                    index++;
	                    stockContent[index1] += getStockTokenString(element);
	                });
	            }
	            else if (result.message) { // no stock and a message - something went wrong
	                appcontroller.appMessageQueue.push({
	                    "message" : {
	                        "nodata" : result.message.substr(0,18)
	                    }
	                });
	                return;
	            }
	            else if (index == 0) { // no stock and no message
	                appcontroller.appMessageQueue.push({
	                    "message" : {
	                        "nodata" : "no stock"
	                    }
	                });
	                return;
	            }

	    	    if (urlIndex == urlArray.length) { // last request finalized - send all stock values to pebble
		    		console.log(JSON.stringify(stockContent));
		    		console.log(stockContent.join(""));
	    	        appcontroller.pushToLocalStorageAndAppMessages(STOCKS_ID, {
	    	            "message" : {
	    	                "stockContent": index + "|" + stockContent.join("") + "e"
	    	            }
	    	        });
	    	    }
	        },
	        function() {
	    		urlIndex++; // got next result

	    		console.log("ERROR during stock request.");

	    	    if (urlIndex == urlArray.length) { // last request finalized - send all stock values to pebble
	    	        appcontroller.pushToLocalStorageAndAppMessages(STOCKS_ID, {
	    	            "message" : {
	    	                "stockContent": index + "|" + stockContent + "e"
	    	            }
	    	        });
	    	    }
	        }
	    );
    });
}

/*
 * fetch currencies operations
 */

function getCurrencyValueString(date, id, rate) {
	var date = date.split("-");

	return id + "|" +
	       Math.round(rate * 10000)/10000 + "|" +
	       date[2] + "." + date[1] + ".|";
}

function fetchCurrencyValues() {
    var url = "https://api.exchangeratesapi.io/latest";

    appcontroller.startJSONRequest(url,
        function(result) {
    		//console.log(JSON.stringify(result));
            var index           = 0;
            var currencyContent = "";
            var currenciesArray = options.currencies.toUpperCase().split(",");

            if (result.date && result.base && result.rates && Object.keys(result.rates).length > 0) {
    	    	var base = result.base;

            	currenciesArray.forEach(function(element, index1, array) {
            		if (element.length != 6) {
            			return; // nothing remains to do
            		}

            		var startCurrency = element.substring(0,3);
            		var endCurrency   = element.substring(3);
            		var exchangeRate  = 0.0;

                    if (startCurrency == base && result.rates.hasOwnProperty(endCurrency)) {
                    	exchangeRate = parseFloat( result.rates[endCurrency] );
                    }
                    else if (endCurrency == base  && result.rates.hasOwnProperty(startCurrency)) {
                    	exchangeRate = 1.0/parseFloat( result.rates[startCurrency] );
                    }
                    else if (result.rates.hasOwnProperty(startCurrency) && result.rates.hasOwnProperty(endCurrency)){
                    	exchangeRate = parseFloat( result.rates[endCurrency] )/parseFloat( result.rates[startCurrency] );
                    }

                    if (exchangeRate > 0.0) {
                        index++;
                        currencyContent += getCurrencyValueString(result.date, element, exchangeRate);
                    }
                });
            }
            else { // no currency
                appcontroller.appMessageQueue.push({
                    "message" : {
                        "nodata" : "no currency"
                    }
                });
                return;
            }

            appcontroller.pushToLocalStorageAndAppMessages(STOCKS_ID, {
                "message" : {
                    "stockContent": index + "|" + currencyContent + "e"
                }
            });
        }
    );
}

function fetchFinancialValues(mode) {
    if (!appcontroller.isRequested(( options.financialMode ? "currencies" : "stock" ),
    		                       options.showFinancials,
    		                       ( options.financialMode ? options.currencies : options.stocks ),
    		                       options.startFinancials, options.stopFinancials, true)) {
        // nothing remains to do
        return;
    }

    switch (options.financialMode) {
	    case 0:
	    	fetchStockValues();
	        break;
	    case 1:
	    	fetchCurrencyValues();
	        break;
        default:
            console.log("Error - received financial mode " + options.financialMode + " is not defined.");
            break;
    }
}

/*
 * individual module interace
 */

module.exports.fetch = fetchFinancialValues;
